@@ -14,14 +14,18 @@
 
 
 import inspect
+import logging
 import os
+import sys
 import time
+import traceback
 import types
 import unittest
 from test_case import MetaTestCase, TestifiedUnitTest
-from test_logger import _log
 from errors import TestifyError
 
+_log = logging.getLogger('testify')
+
 class DiscoveryError(TestifyError): pass
 
 def gather_test_paths(testing_dir):
@@ -76,6 +80,11 @@ def discover(what):
                         test_module = __import__('.'.join(locator.split('.')[:-1]))
                     except (ValueError, ImportError):
                         raise DiscoveryError("Failed to find module %s" % locator)
+            except Exception:
+                raise DiscoveryError("Got unknown error when trying to import %s:\n\n%s" % (
+                    locator,
+                    ''.join(traceback.format_exception(*sys.exc_info()))
+                ))
 
             for part in locator.split('.')[1:]:
                 try:
@@ -96,9 +105,10 @@ def discover(what):
 
         # If it's actually a package, recursively descend.  If it's a true module, import its TestCase members
         elif isinstance(test_module, types.ModuleType):
+            module_suites = suites + getattr(test_module, '_suites', [])
+
             # If it has a __path__, it should be a package (directory)
             if hasattr(test_module, '__path__'):
-                module_suites = getattr(test_module, '_suites', [])
                 module_filesystem_path = test_module.__path__[0]
                 # but let's be sure
                 if os.path.isdir(module_filesystem_path):
@@ -109,22 +119,22 @@ def discover(what):
                             continue
 
                         # If it's actually a package (directory + __init__.py)
-                        if os.path.isdir(os.path.join(module_filesystem_path, item)) and os.path.exists(os.path.join(module_filesystem_path, item, '__init__.py')):
-                            for test_case_class in discover_inner("%s.%s" % (locator, item), suites+module_suites):
+                        item_path = os.path.join(module_filesystem_path, item)
+                        if os.path.isdir(item_path) and os.path.exists(os.path.join(item_path, '__init__.py')):
+                            for test_case_class in discover_inner("%s.%s" % (locator, item), suites=module_suites):
                                 yield test_case_class
 
                         # other than directories, only look in .py files
                         elif item.endswith('.py'):
-                            for test_case_class in discover_inner("%s.%s" % (locator, item[:-3]), suites+module_suites):
+                            for test_case_class in discover_inner("%s.%s" % (locator, item[:-3]), suites=module_suites):
                                 yield test_case_class
 
             # Otherwise it's some other type of module
             else:
-                module_suites = getattr(test_module, '_suites', [])
                 for member_name in dir(test_module):
                     obj = getattr(test_module, member_name)
                     if isinstance(obj, types.TypeType) and inspect.getmodule(obj) == test_module:
-                        for test_case_class in discover_inner(obj, suites + module_suites):
+                        for test_case_class in discover_inner(obj, suites=module_suites):
                             yield test_case_class
 
         # it's not a list, it's not a bare module - let's see if it's an honest-to-god TestCaseBase
@@ -142,13 +152,22 @@ def discover(what):
 
         # detect unittest test cases
         elif issubclass(test_module, unittest.TestCase) and (not '__test__' in test_module.__dict__ or bool(test_module.__test__)):
-            test_case = TestifiedUnitTest.from_unittest_case(test_module)
+            test_case = TestifiedUnitTest.from_unittest_case(test_module, module_suites=suites)
             discover_set.add(test_case)
             yield test_case
 
     discover_set = set()
     time_start = time.time()
+
     for discovery in discover_inner(what):
         yield discovery
+
     time_end = time.time()
     _log.debug("discover: discovered %d test cases in %s" % (len(discover_set), time_end - time_start))
+
+def import_test_class(module_path, class_name):
+    for klass in discover(module_path):
+        if klass.__name__ == class_name:
+            return klass
+
+    raise DiscoveryError(class_name)
