@@ -120,8 +120,8 @@ class TestRunner(object):
         Logging of individual results is accomplished by registering callbacks for
         the TestCase instances to call when they begin and finish running each test.
 
-        At its conclusion, we pass our collected results and to our TestLogger to get
-        testing exceptions and summaries printed out.
+        At its conclusion, we pass our collected results to our TestLogger to
+        print out exceptions and testing summaries.
         """
 
         try:
@@ -145,6 +145,15 @@ class TestRunner(object):
                     test_case.register_callback(test_case.EVENT_ON_RUN_TEST_METHOD, reporter.test_start)
                     test_case.register_callback(test_case.EVENT_ON_COMPLETE_TEST_METHOD, reporter.test_complete)
 
+                    test_case.register_callback(test_case.EVENT_ON_RUN_CLASS_SETUP_METHOD, reporter.class_setup_start)
+                    test_case.register_callback(test_case.EVENT_ON_COMPLETE_CLASS_SETUP_METHOD, reporter.class_setup_complete)
+
+                    test_case.register_callback(test_case.EVENT_ON_RUN_CLASS_TEARDOWN_METHOD, reporter.class_teardown_start)
+                    test_case.register_callback(test_case.EVENT_ON_COMPLETE_CLASS_TEARDOWN_METHOD, reporter.class_teardown_complete)
+
+                    test_case.register_callback(test_case.EVENT_ON_RUN_TEST_CASE, reporter.test_case_start)
+                    test_case.register_callback(test_case.EVENT_ON_COMPLETE_TEST_CASE, reporter.test_case_complete)
+
                 test_case.register_callback(test_case.EVENT_ON_COMPLETE_TEST_METHOD, failure_counter)
 
                 # Now we wrap our test case like an onion. Each plugin given the opportunity to wrap it.
@@ -169,13 +178,15 @@ class TestRunner(object):
         suites = defaultdict(list)
         for test_instance in self.discover():
             for test_method in test_instance.runnable_test_methods():
-                for suite_name in test_method._suites:
+                for suite_name in test_instance.suites(test_method):
                     suites[suite_name].append(test_method)
         suite_counts = dict((suite_name, "%d tests" % len(suite_members)) for suite_name, suite_members in suites.iteritems())
 
         pp = pprint.PrettyPrinter(indent=2)
         print(pp.pformat(dict(suite_counts)))
 
+        return suite_counts
+
     def list_tests(self, selected_suite_name=None):
         """Lists all tests, optionally scoped to a single suite."""
         test_list = []
@@ -187,4 +198,6 @@ class TestRunner(object):
         pp = pprint.PrettyPrinter(indent=2)
         print(pp.pformat([self.get_test_method_name(test) for test in test_list]))
 
+        return test_list
+
 # vim: set ts=4 sts=4 sw=4 et:
